/*********************************************************

                        mkr0

**********************************************************

mkr0 fills a simulation box of dimensions provided by the user with 
a number of equally distributed and randomly oriented solvent molecules.
Optionally molecules that overlap with coordinates of a solute, as 
provided in a pdb file, can be discarded. 
The skew start method used to generate an array of molecules that are 
equally separated but do not represent a crystal lattice was origanally 
devised by Keith Refson. It is described in the moldy manual in
http://www.earth.ox.ac.uk/~keithr/moldy-manual/node16.html
The random rotation subroutine is from:
http://www.materialography.de/Book/sourcecodes/

limitations:

- only PDB files with ATOM or HETATM entries in the format as specified 
  in http://www.rcsb.org/pdb/docs/format/pdbguide2.2/guide2.2_frame.html
  are supported by mkr0
- only rectangular boxes are supported
- ...

please report bugs to:

Author: Michael Brunsteiner
mbx0009 [at] yahoo.com
http://www.ucl.ac.uk/~uccambr

*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "./micutc.h"

void randrot(int,double*,double*,double*);
double randd(void);

void error_msg(char*);

int main(int ARGC, char* ARGV[]) {

    char** solventlab;
    char** solventresid;
    char*  solventchainid;

    char** solutelab;
    char** soluteresid;
    char*  solutechainid;
    int*   soluteresnr;

    char xstring[9];
    char ystring[9];
    char zstring[9];

    char* strptr;
    char tmpstr[256];
    char line[256];
    char tmplab[256];
    char tmpresid[256];
    char tmpchainid[256];
    char sdum[256];
    int  tmpresnr;
    char ATOM[] = "ATOM\0";
    char HETATM[] = "HETATM\0";


    int NMsolvent,i,j,k,l,COORDS,nb,TEST,XPBC,YPBC,ZPBC, SOLUTE,Noverlap,
      OUTFILE,CENTER,n,nr,na,len,NAsolvent,NAsolute,idum,napm,oops,RANDROT,
      nposions,nnegions,npiassigned,nniassigned,remote_solvent_idx,nchecked;

    double boxx,boxy,boxz,x,y,z,c,sqOLdist,d_min,halfboxx,halfboxy,halfboxz,
           dx,dy,dz,sx,sy,sz,NpLx,NpLy,NpLz,min,dis,xsum,ysum,zsum,scal,
           min_ion_dist,min_ion_sqdist,shortest_sqdist,d2,longest_sqdist_so_far;
    double* ddum;
    char* read;

    double** RX;
    double** RY;
    double** RZ;
    double* RSx;
    double* RSy;
    double* RSz;
    double* tmprsx;
    double* tmprsy;
    double* tmprsz;
    double* xsolute;
    double* ysolute;
    double* zsolute;
    double* xpion;
    double* ypion;
    double* zpion;
    double* xnion;
    double* ynion;
    double* znion;

    FILE *fin, *fout, *fsolute;

    int *OVERLAP;

/***** initialisation stuff ******/

    COORDS    = 0;
    CENTER    = 0;
    SOLUTE    = 0;
    OUTFILE   = 0;
    TEST      = 0;
    RANDROT   = 0;
    NMsolvent = 0;
    Noverlap  = 0;
    nb        = 0;
    XPBC      = 0;
    YPBC      = 0;
    ZPBC      = 0;
    NAsolvent = 1;
    NAsolute  = 0;
    NMsolvent = 0;
    nposions  = 0;
    nnegions  = 0;

    min_ion_dist = 4.0;
    d_min     = 2.5;
    sqOLdist  = d_min*d_min;    

/***** ... analyse the command line arguments ... *****/

    for(i=1;i<ARGC;i++) {
        
        if(ARGV[i][0]!='-') error_msg("wrong format in command line");
        else switch(ARGV[i][1]) {
        
            case 'n': NMsolvent = atoi(ARGV[++i]);
                      break;
            case 'f': COORDS = ++i;
                      break;
            case 't': TEST=1;
                      break;
            case 'd': boxx = atof(ARGV[++i]);
                      boxy = atof(ARGV[++i]);
                      boxz = atof(ARGV[++i]);
                      break;
            case 'x': XPBC=1;
                      break;
            case 'y': YPBC=1;
                      break;
            case 'z': ZPBC=1;
                      break;
            case 's': SOLUTE = ++i;
                      break;
            case 'o': OUTFILE = ++i;
                      break;
            case 'c': d_min = atof(ARGV[++i]);
                      sqOLdist = d_min*d_min;
                      break;
            case '0': CENTER=1;
                      break;
            case 'm': nnegions = atoi(ARGV[++i]);
                      break;
            case 'p': nposions = atoi(ARGV[++i]);
                      break;
	    case 'r': RANDROT=1;
                      break;
            default: error_msg("unrecognized command line parameter");
        }
    }

    if(NMsolvent<=0||boxx<=0.0||boxy<=0.0||boxz<=0.0||OUTFILE==0)
      error_msg("check command line parameters");

/***** open files and count atoms *****/

    if((fout=fopen(ARGV[OUTFILE], "w"))==NULL) {
      printf("can't open output file ...\n");
      exit(1);
    } else fprintf(stderr,"opened output PDB file %s\n", ARGV[OUTFILE]);

    if(COORDS) {
    
        if((fin=fopen(ARGV[COORDS],"r"))==NULL) {
            printf("can't see no %s\n", ARGV[COORDS]);
            exit(1);
        }

        NAsolvent=0;

        while (read = fgets(line, 256, fin)) {
          if(strncmp(line,ATOM,4)||strncmp(line,HETATM,4)) NAsolvent++;
        }

        fprintf(stderr,"opened %s containing %d atoms\n",  ARGV[COORDS],NAsolvent);
        rewind(fin);
    }

    if(SOLUTE) {

        if((fsolute=fopen(ARGV[SOLUTE],"r"))==NULL) {
            printf("can't see no %s\n", ARGV[SOLUTE]);
            exit(1);
        }


        while (read = fgets(line, 256, fsolute)) {
          if(strncmp(line,ATOM,4)==0 || strncmp(line,HETATM,4)==0) NAsolute++;
        }

        fprintf(stderr,"read %s containing %d atoms\n", ARGV[SOLUTE],NAsolute);
        rewind(fsolute);
    }

/***** allocate memory and initialize vars *****/

    solventresid = (char**)c0mat(NAsolvent,5);
    solventlab = (char**)c0mat(NAsolvent,5);
    solventchainid = (char*)c0vec(NAsolvent);
    soluteresid = (char**)c0mat(NAsolute,5);
    solutelab = (char**) c0mat(NAsolute,5);
    solutechainid =(char*)c0vec(NAsolute);
    soluteresnr = (int*)i0vec(NAsolute);

    RX         = (double**)d0mat(NMsolvent,NAsolvent);
    RY         = (double**)d0mat(NMsolvent,NAsolvent);
    RZ         = (double**)d0mat(NMsolvent,NAsolvent);
    OVERLAP    = (int*)i0vec(NMsolvent);
    RSx        = (double*)d0vec(NAsolvent);
    RSy        = (double*)d0vec(NAsolvent);
    RSz        = (double*)d0vec(NAsolvent);
    tmprsx     = (double*)d0vec(NAsolvent);
    tmprsy     = (double*)d0vec(NAsolvent);
    tmprsz     = (double*)d0vec(NAsolvent);
    xsolute    = (double*)d0vec(NAsolute);
    ysolute    = (double*)d0vec(NAsolute);
    zsolute    = (double*)d0vec(NAsolute);
    xpion      = (double*)d0vec(nposions);
    ypion      = (double*)d0vec(nposions);
    zpion      = (double*)d0vec(nposions);
    xnion      = (double*)d0vec(nnegions);
    ynion      = (double*)d0vec(nnegions);
    znion      = (double*)d0vec(nnegions);

    if(NAsolvent==1) {
      solventlab[0][0] = 'A';
      solventlab[0][1] = 'r';
      solventlab[0][2] = '\0';
      solventresid[0][0] = 'a';
      solventresid[0][1] = 'r';
      solventresid[0][2] = 'g';
      solventresid[0][3] = '\0';
      solventchainid[0] = 'S';
    }

    for(i=0;i<NAsolvent;i++) {
        RSx[i] = 0.0;
        RSy[i] = 0.0;
        RSz[i] = 0.0;
    }

    for(i=0;i<NAsolute;i++) {
        xsolute[i] = 0.0;
        ysolute[i] = 0.0;
        zsolute[i] = 0.0;
    }

    for(i=0;i<NMsolvent;i++) {
        OVERLAP[i] = 0;
    }

    if(CENTER) {
      halfboxx=0.0;
      halfboxy=0.0;
      halfboxz=0.0;
    } else {
      halfboxx=boxx/2.0;
      halfboxy=boxy/2.0;
      halfboxz=boxz/2.0;
    }

/***** read the relative coordinates of the atomic sites *****/

   if(COORDS) {

     i=0;
     xsum = 0.0;
     ysum = 0.0;
     zsum = 0.0;

     while(read = fgets(line, 256, fin)) {
       if(strncmp(line,ATOM,4)||strncmp(line,HETATM,4)) {

         strncpy(solventlab[i],&(line[12]),4);
         strncpy(solventresid[i],&(line[17]),4);
         solventchainid[i] = line[21];
	 strncpy(xstring,&(line[30]),8);
	 strncpy(ystring,&(line[38]),8);
	 strncpy(zstring,&(line[46]),8);
	 solventlab[i][4] = '\0';
         solventresid[i][4] = '\0';
         xstring[8] = '\0';
         ystring[8] = '\0';
         zstring[8] = '\0';
         RSx[i] = atof(xstring);
         RSy[i] = atof(ystring);
         RSz[i] = atof(zstring);
         xsum+=RSx[i];
         ysum+=RSy[i];
         zsum+=RSz[i];

         i++;
       }
     }

/***** move geometric center to the origin *****/

     scal = 1.0/(double)i;
     xsum *= scal;
     ysum *= scal;
     zsum *= scal;

     for(j=0;j<=i;j++) {
       RSx[j] -= xsum;
       RSy[j] -= ysum;
       RSz[j] -= zsum;
     }
   }


   if(SOLUTE) {

     i=0;
     xsum = 0.0;
     ysum = 0.0;
     zsum = 0.0;

     while (read = fgets(line, 256, fsolute)) {
       if(strncmp(line,ATOM,4)==0 || strncmp(line,HETATM,4)==0) {

         strncpy(solutelab[i],&(line[12]),4);
         strncpy(soluteresid[i],&(line[17]),4);
         solutechainid[i] = line[21];
	 strncpy(sdum,&(line[22]),4);
	 strncpy(xstring,&(line[30]),8);
	 strncpy(ystring,&(line[38]),8);
	 strncpy(zstring,&(line[46]),8);
	 solutelab[i][4] = '\0';
         soluteresid[i][4] = '\0';
         sdum[4] = '\0';
         xstring[8] = '\0';
         ystring[8] = '\0';
         zstring[8] = '\0';
	 soluteresnr[i] = atoi(sdum);
         xsolute[i] = atof(xstring);
         ysolute[i] = atof(ystring);
         zsolute[i] = atof(zstring);
         xsum+=xsolute[i];
         ysum+=ysolute[i];
         zsum+=zsolute[i];

         i++;
       }
     }

/***** move geometric center to the origin *****/

     scal = 1.0/(double)i;
     xsum *= scal;
     ysum *= scal;
     zsum *= scal;

     for(j=0;j<=i;j++) {
       xsolute[j] = xsolute[j] - xsum + halfboxx;
       ysolute[j] = ysolute[j] - ysum + halfboxy;
       zsolute[j] = zsolute[j] - zsum + halfboxz;
     }
   }

/***** determine the distance between layers for each dimension *****/

    c = pow(((double)NMsolvent)/boxx/boxy/boxz,(1.0/3.0));
    NpLx = c*boxx;
    NpLy = c*boxy;
    NpLz = c*boxz;

    dx = boxx/NpLx;
    dy = boxy/NpLx/NpLy;
    dz = boxz/NpLx/NpLy/NpLz;

    x = -boxx*0.5;
    y = -boxy*0.5;
    z = -boxz*0.5;

/***** calculate the coordinates *****/

    for(i=0;i<NMsolvent;i++) {

        x += dx;
        y += dy;
        z += dz;

        if(x>boxx*0.5) x -= boxx;
        if(y>boxy*0.5) y -= boxy;
        if(z>boxz*0.5) z -= boxz;
        if(x<-boxx*0.5) x += boxx;
        if(y<-boxy*0.5) y += boxy;
        if(z<-boxz*0.5) z += boxz;

        for(j=0;j<NAsolvent;j++) {
          tmprsx[j] = RSx[j];
          tmprsy[j] = RSy[j];
          tmprsz[j] = RSz[j];
	}

	if(RANDROT==1) { randrot(NAsolvent,tmprsx,tmprsy,tmprsz); }

            for(j=0;j<NAsolvent;j++) {
                
                sx = x+tmprsx[j];
                sy = y+tmprsy[j];
                sz = z+tmprsz[j];

                if(XPBC) {
                  if(sx >  boxx*0.5) sx -= boxx;
                  if(sx < -boxx*0.5) sx += boxx;
		}
                if(YPBC) {
                  if(sy >  boxy*0.5) sy -= boxy;
                  if(sy < -boxy*0.5) sy += boxy;
		}
                if(ZPBC) {
                  if(sz >  boxz*0.5) sz -= boxz;
                  if(sz < -boxz*0.5) sz += boxz;
		}

                RX[i][j] = sx+halfboxx;
                RY[i][j] = sy+halfboxy;
                RZ[i][j] = sz+halfboxz;
            }
    }

/***** find which solvent molecules overlap the solute and exclude them *****/

    if(SOLUTE) {

      for(i=0;i<NMsolvent;i++) {
	
        for(j=0;j<NAsolvent;j++) {
	  
          for(k=0;k<NAsolute;k++) {

            dx = RX[i][j]-xsolute[k];
            dy = RY[i][j]-ysolute[k];
            dz = RZ[i][j]-zsolute[k];

            if(dx> boxx*0.5)  dx -= boxx;
            if(dx<-boxx*0.5)  dx += boxx;
            if(dy> boxy*0.5)  dy -= boxy;
            if(dy<-boxy*0.5)  dy += boxy;
            if(dz> boxz*0.5)  dz -= boxz;
            if(dz<-boxz*0.5)  dz += boxz;

            if(sqOLdist > dx*dx+dy*dy+dz*dz) OVERLAP[i] = 1;

	  }

	}

        if(OVERLAP[i]) Noverlap ++;
      
      }
    }

/***** replace some waters by positive ions ******************************************/

    npiassigned = 0;
    nniassigned = 0;

    if(nposions>0) {

      while( npiassigned<nposions ) {

	shortest_sqdist=1000000000.0;
        longest_sqdist_so_far = 0.0;

        for(i=0;i<NMsolvent;i++) {

          if(OVERLAP[i] == 0) {

	    shortest_sqdist=1000000000.0;

            for(k=0;k<NAsolute;k++) {

              dx = RX[i][0]-xsolute[k];
              dy = RY[i][0]-ysolute[k];
              dz = RZ[i][0]-zsolute[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) shortest_sqdist=d2;
	    }

            for(k=0;k<npiassigned;k++) {

              dx = RX[i][0]-xpion[k];
              dy = RY[i][0]-ypion[k];
              dz = RZ[i][0]-zpion[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) shortest_sqdist=d2;
	    }

            for(k=0;k<nniassigned;k++) {

              dx = RX[i][0]-xnion[k];
              dy = RY[i][0]-ynion[k];
              dz = RZ[i][0]-znion[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) { shortest_sqdist=d2;}
	    }
	  }
	
          if(shortest_sqdist>longest_sqdist_so_far && OVERLAP[i]==0) {
            remote_solvent_idx=i;
            longest_sqdist_so_far=shortest_sqdist;
          }
	}

        OVERLAP[remote_solvent_idx]=1;
	shortest_sqdist=1000000000.0;
        xpion[npiassigned]=RX[remote_solvent_idx][0];
        ypion[npiassigned]=RY[remote_solvent_idx][0];
        zpion[npiassigned]=RZ[remote_solvent_idx][0];

        printf("+ chose solvent %8d, closest distance: %.4f",remote_solvent_idx,sqrt(longest_sqdist_so_far));
        printf(" at: %10.3f%10.3f%10.3f\n", xpion[npiassigned],ypion[npiassigned],zpion[npiassigned]);

        longest_sqdist_so_far = 0.0;
        npiassigned++;
      }
    }


/***** replace some waters by negative ions ******************************************/

    if(nnegions>0) {

      while( nniassigned<nnegions ) {

	shortest_sqdist=1000000000.0;
        longest_sqdist_so_far = 0.0;

        for(i=0;i<NMsolvent;i++) {

          if(OVERLAP[i] == 0) {

	    shortest_sqdist=1000000000.0;

            for(k=0;k<NAsolute;k++) {

              dx = RX[i][0]-xsolute[k];
              dy = RY[i][0]-ysolute[k];
              dz = RZ[i][0]-zsolute[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) shortest_sqdist=d2;
	    }

            for(k=0;k<npiassigned;k++) {

              dx = RX[i][0]-xpion[k];
              dy = RY[i][0]-ypion[k];
              dz = RZ[i][0]-zpion[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) shortest_sqdist=d2;
	    }

            for(k=0;k<nniassigned;k++) {

              dx = RX[i][0]-xnion[k];
              dy = RY[i][0]-ynion[k];
              dz = RZ[i][0]-znion[k];

              if(dx> boxx*0.5)  dx -= boxx;
              if(dx<-boxx*0.5)  dx += boxx;
              if(dy> boxy*0.5)  dy -= boxy;
              if(dy<-boxy*0.5)  dy += boxy;
              if(dz> boxz*0.5)  dz -= boxz;
              if(dz<-boxz*0.5)  dz += boxz;

              d2 = dx*dx+dy*dy+dz*dz;

              if(d2<shortest_sqdist) { shortest_sqdist=d2;}
	    }
	  }
	
          if(shortest_sqdist>longest_sqdist_so_far && OVERLAP[i]==0) {
            remote_solvent_idx=i;
            longest_sqdist_so_far=shortest_sqdist;
          }
	}

        OVERLAP[remote_solvent_idx]=1;
	shortest_sqdist=1000000000.0;
        xnion[nniassigned]=RX[remote_solvent_idx][0];
        ynion[nniassigned]=RY[remote_solvent_idx][0];
        znion[nniassigned]=RZ[remote_solvent_idx][0];

        printf("- chose solvent %8d, closest distance: %.4f",remote_solvent_idx,sqrt(longest_sqdist_so_far));
        printf(" at: %10.3f%10.3f%10.3f\n", xnion[nniassigned],ynion[nniassigned],znion[nniassigned]);

        longest_sqdist_so_far = 0.0;
        nniassigned++;
      }
    }


/***** find the minimum distance between two atoms in the resulting config. *****/

    if(TEST) {

    min = 1.0e+30;

      for(i=0;i<NMsolvent-1;i++)
        for(j=i+1;j<NMsolvent;j++)
                for(k=0;k<NAsolvent;k++)
                    for(l=0;l<NAsolvent;l++) { 

            dx = RX[i][k]-RX[j][l];
            dy = RY[i][k]-RY[j][l];
            dz = RZ[i][k]-RZ[j][l];

            if(dx> boxx) dx -= boxx;
            if(dx<-boxx) dx += boxx;
            if(dy> boxy) dy -= boxy;
            if(dy<-boxy) dy += boxy;
            if(dz> boxz) dz -= boxz;
            if(dz<-boxz) dz += boxz;

           dis = sqrt(dx*dx+dy*dy+dz*dz);
           if(dis<min) min = dis;

      }

    }

/***** finally print out the coords *****/

    n  = 0;
    nr = 0;
    na = 0;

    fprintf(fout,"REMARK solvent box generated by mkr0\n");
    fprintf(fout,"REMARK\n");
    fprintf(fout,"CRYST1%9.3lf%9.3lf%9.3lf  90.00  90.00  90.00 P 1           1\n",boxx,boxy,boxz);

    for(i=0;i<NAsolute;i++) {
      nb += fprintf(fout,"ATOM%7d %4s %4s%c%4d%12.3lf%8.3lf%8.3lf  0.00  0.00\n", 
		    na+1,solutelab[na],soluteresid[na],solutechainid[na],soluteresnr[na],
                    xsolute[na],ysolute[na],zsolute[na]);
      na++;
    }

    if(SOLUTE) nr = soluteresnr[na-1]; else nr = 0;

    for(i=0;i<nposions;i++) {
      nr++;
      nb += fprintf(fout,"ATOM%7d %4s %4s%c%4d%12.3lf%8.3lf%8.3lf  0.00  0.00\n", 
		    na+1,"Na  ","Na+ ",' ',nr, xpion[i],ypion[i],zpion[i]);
      na++;
    }

    for(i=0;i<nnegions;i++) {
      nr++;
      nb += fprintf(fout,"ATOM%7d %4s %4s%c%4d%12.3lf%8.3lf%8.3lf  0.00  0.00\n", 
		    na+1,"Cl  ","Cl- ",' ',nr, xnion[i],ynion[i],znion[i]);
      na++;
    }

    for(i=0;i<NMsolvent;i++)

      if(!OVERLAP[i]) {
	napm=0;
        nr++;
        for(j=0;j<NAsolvent;j++) {
          nb += fprintf(fout,"ATOM%7d %4s %4s%5d%12.3lf%8.3lf%8.3lf  0.00  0.00\n", 
                na+1,solventlab[napm],solventresid[napm],nr,
                RX[i][j],RY[i][j],RZ[i][j]);
          na++;
          napm++;
      }
    }

    fprintf(stderr,"%d bytes written in %s\n",nb,ARGV[OUTFILE]);
    if(TEST) fprintf(stderr,"minimum distance = %lf\n",min);
    if(SOLUTE) fprintf(stderr,"%d solvent molecules are left\n",NMsolvent-Noverlap);

    fclose(fout);
    if(COORDS) fclose(fin);
    if(SOLUTE) fclose(fsolute);

    return(NMsolvent-Noverlap);
}



void error_msg(char* msg) {

    fprintf(stderr,"\n%s\n\n", msg);
    fprintf(stderr,"usage: mkr0 -n N -d x y z -o ofile\n\n");
    fprintf(stderr,"  N is the total number of solvent molecules,\n");
    fprintf(stderr,"  x y and z are the box dimensions,\n");
    fprintf(stderr,"  ofile is the name of the output file.\n\n");
    fprintf(stderr,"Optional arguments:\n");
    fprintf(stderr,"  -f filename\n");
    fprintf(stderr,"    filename is a file, containing the geometry of\n");
    fprintf(stderr,"    a single solvent molecule. If this parameter is not given\n");
    fprintf(stderr,"    the molecule is assumed to be a single atom\n");
    fprintf(stderr,"  -s filename\n");
    fprintf(stderr,"    filename is a file, containing the coordinates of (a) solute molecule(s)\n");
    fprintf(stderr,"    The output of each of the N created molecules that is closer to\n");
    fprintf(stderr,"    any solute molecule than d_min (default: 2.5 A) will be surpressed\n");
    fprintf(stderr,"  -c d_min\n");
    fprintf(stderr,"    d_min is the minimum distance (in Angstrom) between solute and solvent molecules\n");
    fprintf(stderr,"  -x -y -z\n");
    fprintf(stderr,"    Apply PBC to atoms (instead of molecules) in the given directions\n");
    fprintf(stderr,"  -m n\n");
    fprintf(stderr,"    replace n solvent atoms by Cl-\n");
    fprintf(stderr,"  -p n\n");
    fprintf(stderr,"    replace n solvent atoms by Na+\n");
    fprintf(stderr,"  -t\n");
    fprintf(stderr,"    Print out the smallest distance between two solvent atoms in the configuration\n");
    fprintf(stderr,"  -0\n");
    fprintf(stderr,"    Box is centered at the origin.\n");
    fprintf(stderr,"    By default the box is centered at x/2,y/2,z/2\n");
    fprintf(stderr,"  -r\n");
    fprintf(stderr,"    rotate solvent molecules randomly before inserting at lattice\n\n");
    exit(1);
}


void randrot(int n, double *x, double *y, double *z) {

   int i;
   double phi, Phi, xi;
   double cos_phi, sin_phi, cos_Phi, sin_Phi, eta;
   double u, v, w;
   double a[3][3];

   phi=2.*M_PI*randd();
   Phi=2.*M_PI*randd();
   xi=2.*randd()-1.;

   cos_phi=cos(phi); sin_phi=sin(phi);
   cos_Phi=cos(Phi); sin_Phi=sin(Phi);
   eta=sqrt(1.-xi*xi);

   a[0][0]=cos_Phi*cos_phi-xi*sin_Phi*sin_phi;
   a[0][1]=cos_Phi*sin_phi+xi*sin_Phi*cos_phi;
   a[0][2]=eta*sin_Phi;
   a[1][0]= -sin_Phi*cos_phi-xi*cos_Phi*sin_phi;
   a[1][1]= -sin_Phi*sin_phi+xi*cos_Phi*cos_phi;
   a[1][2]=eta*cos_Phi;
   a[2][0]=eta*sin_phi;
   a[2][1]= -eta*cos_phi;
   a[2][2]=xi;

   for(i=0;i<n;i++)
   {  u=x[i]; v=y[i]; w=z[i];
      x[i]=a[0][0]*u+a[0][1]*v+a[0][2]*w;
      y[i]=a[1][0]*u+a[1][1]*v+a[1][2]*w;
      z[i]=a[2][0]*u+a[2][1]*v+a[2][2]*w;
   }
}

double randd(void) {
  return (double)random()/(double)RAND_MAX;
}
